package de.hdm_stuttgart.mi.sd1.task1;

/**
 * Zipping up two arrays into one.
 */
public class _5_ArrayZipper {

  /**
   *
   * <p>“Zipping up” two arrays into a single one. If present the “left” array's first value serves as the starting
   * point. Examples:</p>
   *
   * <table border="1" style="border-collapse: collapse;">
   *   <caption>Two arrays of equal length.</caption>
   *   <tr>
   *     <td>
   *   <pre>{<span style="color:red;">3</span>, <span
   *         style="color:red;">6</span>, <span style="color:red;">1</span>, <span
   *         style="color:red;">5</span>}
   *  ➘➚ ➘➚ ➘➚ ➘    ==&gt; {<span style="color:red;">3</span>, <span
   *         style="color:green;">9</span>, <span style="color:red;">6</span>, <span
   *         style="color:green;">0</span>, <span style="color:red;">1</span>, <span
   *         style="color:green;">4</span>, <span style="color:red;">5</span>, <span
   *         style="color:green;">7</span>}
   *  {<span style="color:green;">9</span>, <span style="color:green;">0</span>, <span
   *         style="color:green;">4</span>, <span style="color:green;">7</span>}</pre>
   *     </td>
   *     <td> like </td>
   *     <td>
   *       <img src="doc-files/zipper.svg" alt="Zipper image" width="150" >
   *     </td>
   *
   *   </tr>
   *
   * </table>
   *
   *  <table border="1" style="border-collapse: collapse;">
   *     <caption>Second array having two more elements: Excess values printed in
   *    *  <span style="color:blue;">blue</span> are being appended.</caption>
   *       <tr>
   *         <td><pre>{<span style="color:red;">3</span>, <span style="color:red;">6</span>}
   *  ➘➚ ➘  ↷  ↷    ==&gt; {<span style="color:red;">3</span>, <span
   *               style="color:green;">9</span>, <span style="color:red;">6</span>, <span
   *               style="color:green;">0</span>, <span style="color:blue;">4</span>, <span
   *               style="color:blue;">7</span>}
   *  {<span style="color:green;">9</span>, <span style="color:green;">0</span>, <span
   *               style="color:blue;">4</span>, <span style="color:blue;">7</span>}</pre></td>
   *
   *         <td><pre>{<span style="color:red;">3</span>, <span style="color:red;">6</span>, <span
   *               style="color:red;">1</span>, <span style="color:blue;">5</span>}
   *  ➘➚ ➘➚ ⤻      ==&gt; {<span style="color:red;">3</span>, <span
   *               style="color:green;">9</span>, <span style="color:red;">6</span>, <span
   *               style="color:green;">0</span>, <span style="color:red;">1</span>, <span
   *               style="color:blue;">5</span>}
   *  {<span style="color:green;">9</span>, <span style="color:green;">0</span>}</pre></td>
   *       </tr>
   *     </table>
   *
   *   <table border="1" style="border-collapse: collapse;">
   *     <caption><code>null</code> values being handled like empty arrays:</caption>
   *       <tr>
   *         <td><pre><code>null</code> or {}
   *   ↷  ↷  ↷      ==&gt; {<span style="color:green;">9</span>, <span
   *               style="color:blue;">0</span>, <span style="color:blue;">4</span>, <span
   *               style="color:blue;">7</span>}
   *{<span style="color:green;">9</span>, <span style="color:blue;">0</span>, <span
   *               style="color:blue;">4</span>, <span style="color:blue;">7</span>}</pre></td>
   *
   *         <td><pre>{<span style="color:red;">3</span>, <span style="color:blue;">6</span>, <span
   *               style="color:blue;">1</span>, <span style="color:blue;">5</span>}
   *  ⤻  ⤻  ⤻     ==&gt; {<span style="color:red;">3</span>, <span
   *               style="color:blue;">6</span>, <span style="color:blue;">1</span>, <span
   *               style="color:blue;">5</span>}
   *<code>null</code> or {}</pre></td>
   *
   *         <td><pre><code>null</code>  or {}
   *                ==&gt; {}
   *<code>null</code> or {}</pre></td>
   *       </tr>
   *     </table>
   *
   * @param left First array or <code>null</code>. The zipping process will start with this array's first value if present.
   * @param right Second array or <code>null</code>.
   * @return Both arrays' combined values according to the zipping description. The resulting array's length
   *         equals the sum of lengths of both contributors.
   *
   * <section class="implementationHints">
   *    <h4>Possible implementation idea:</h4>
   *    <ol>
   *        <li>Handle <code>null</code> array argument values beforehand.</li>
   *        <li>Create a new array having the combined length of both input arrays.</li>
   *        <li>Get the minimum length of both input arrays.</li>
   *        <li>Use a loop till minimum length to combine both arrays</li>
   *        <li>Add the longer array's excess values.</li>
   *    </ol>
   * </section>
   */
  static public int[] zipper(int[] left, final int[] right) {

    // First handle possible null values
    if (null == left && null == right) {    // Both arrays are null
      return new int[0];
    } else if(null == left) {               // Only left array is null
      return right;
    } else if (null == right) {             // Only right array is null
      return left;
    } else {                                // Both arrays differ from null

      // Find longer of both arrays
      final int[] longerArray ;
      final int maxLength, minLength;

      if (left.length < right.length) {
        longerArray = right;
        maxLength = right.length;
        minLength = left.length;
      } else {
        longerArray = left;
        maxLength = left.length;
        minLength = right.length;
      }

      final int[] result = new int[left.length + right.length];

      // Zipping common length parts
      for (int i = 0; i < minLength; i++) {
        result[2 * i]     = left[i];
        result[2 * i + 1] = right[i];
      }

      // Append longer array's excess values.
      System.arraycopy(longerArray, minLength, result, 2 * minLength, maxLength - minLength);

      return result;
    }
  }

  private _5_ArrayZipper(){/* Ignore me: Suppressing default constructor javadoc generation:
                              javadoc does not include private constructors by default */}
}
